==> cmlapi-8.x-1.0.tar.gz: cmlapi/src/Service/Scheme.php <==
<?php

namespace Drupal\cmlapi\Service;

/**
 * Scheme from 1c.
 */
class Scheme implements SchemeInterface {

  /**
   * Features.
   *
   * @var array
   */
  protected $feature;

  /**
   * Taxonomy.
   *
   * @var array
   */
  protected $taxonomy;

  /**
   * Catalog Parser.
   *
   * @var \Drupal\cmlapi\Service\ParserCatalog
   */
  protected $catalogParcer;

  /**
   * Xml Parser.
   *
   * @var \Drupal\cmlapi\Service\XmlParserInterface
   */
  protected $xmlParser;

  /**
   * Constructs a new Scheme object.
   *
   * @param \Drupal\cmlapi\Service\ParserCatalog $catalog
   *   CAtalog Parser.
   * @param \Drupal\cmlapi\Service\XmlParserInterface $xml_parser
   *   XML Parcer.
   */
  public function __construct(
    ParserCatalog $catalog,
    XmlParserInterface $xml_parser
  ) {
    $this->feature = [];
    $this->taxonomy = [];
    $this->catalogParcer = $catalog;
    $this->xmlParser = $xml_parser;
  }

  /**
   * Page tree.
   */
  public function init(int $cml) {
    $data = $this->catalogParcer->parse($cml);
    $this->featureList($data);
    $category = $this->categoryList($data);
    return [
      'category' => $category,
      'feature' => $this->feature,
      'taxonomy' => $this->taxonomy,
    ];
  }

  /**
   * Category.
   */
  private function categoryList($data) {
    $category = [];
    if ($data && isset($data['category'])) {
      $cats = $this->xmlParser->arrayNormalize($data['category']);
      foreach ($cats as $cat) {
        $id = $cat['Ид'];
        $category[$id] = [
          'id' => $id,
          'name' => $cat['Наименование'],
          'props' => $this->categoryProps($cat),
        ];
      }
    }
    return $category;
  }

  /**
   * Props.
   */
  private function categoryProps($cat) {
    $props = [];
    if (!empty($cat['Свойства']['Ид'])) {
      foreach ($cat['Свойства']['Ид'] as $pid) {
        if (isset($this->feature[$pid])) {
          $props[$pid] = $this->feature[$pid];
        }
        else {
          $props[$pid] = $pid;
        }
      }
    }
    return $props;
  }

  /**
   * Features.
   */
  private function featureList($data) {
    if ($data && !empty($data['feature'])) {
      $features = $this->xmlParser->arrayNormalize($data['feature']);
      foreach ($features as $value) {
        $id = $value['Ид'];
        $type = $value['ТипЗначений'];
        $feature = [
          'id' => $id,
          'name' => $value['Наименование'],
          'type' => $value['ТипЗначений'],
          'product' => $value['ДляТоваров'],
          'props' => $this->featureProps($value),
        ];
        if ($type == 'Справочник') {
          $this->taxonomy[$id] = $feature;
        }
        else {
          $this->feature[$id] = $feature;
        }
      }
    }
  }

  /**
   * Feature Props.
   */
  private function featureProps($feature) {
    $props = [];
    if (!empty($feature['ВариантыЗначений']['Справочник'])) {
      $props = $this->xmlParser->xml2KeyVal($feature['ВариантыЗначений']['Справочник']);
    }
    return $props;
  }

}

==> cmlapi-8.x-1.1.tar.gz: cmlapi/src/Service/Scheme.php <==
<?php

namespace Drupal\cmlapi\Service;

/**
 * Scheme from 1c.
 */
class Scheme implements SchemeInterface {

  /**
   * Features.
   *
   * @var array
   */
  protected $feature;

  /**
   * Taxonomy.
   *
   * @var array
   */
  protected $taxonomy;

  /**
   * Catalog Parser.
   *
   * @var \Drupal\cmlapi\Service\ParserCatalog
   */
  protected $catalogParcer;

  /**
   * Xml Parser.
   *
   * @var \Drupal\cmlapi\Service\XmlParserInterface
   */
  protected $xmlParser;

  /**
   * Constructs a new Scheme object.
   *
   * @param \Drupal\cmlapi\Service\ParserCatalog $catalog
   *   CAtalog Parser.
   * @param \Drupal\cmlapi\Service\XmlParserInterface $xml_parser
   *   XML Parcer.
   */
  public function __construct(
    ParserCatalog $catalog,
    XmlParserInterface $xml_parser
  ) {
    $this->feature = [];
    $this->taxonomy = [];
    $this->catalogParcer = $catalog;
    $this->xmlParser = $xml_parser;
  }

  /**
   * Page tree.
   */
  public function init(int $cml) {
    $data = $this->catalogParcer->parse($cml);
    dsm($data);
    $this->featureList($data);
    $category = $this->categoryList($data);
    return [
      'category' => $category,
      'feature' => $this->feature,
      'taxonomy' => $this->taxonomy,
    ];
  }

  /**
   * Category.
   */
  private function categoryList($data) {
    $category = [];
    if ($data && isset($data['category']) && $data['category']) {
      $cats = $this->xmlParser->arrayNormalize($data['category']);
      foreach ($cats as $cat) {
        $id = $cat['Ид'];
        $category[$id] = [
          'id' => $id,
          'name' => $cat['Наименование'],
          'props' => $this->categoryProps($cat),
        ];
      }
    }
    return $category;
  }

  /**
   * Props.
   */
  private function categoryProps($cat) {
    $props = [];
    if (!empty($cat['Свойства']['Ид'])) {
      foreach ($cat['Свойства']['Ид'] as $pid) {
        if (isset($this->feature[$pid])) {
          $props[$pid] = $this->feature[$pid];
        }
        else {
          $props[$pid] = $pid;
        }
      }
    }
    return $props;
  }

  /**
   * Features.
   */
  private function featureList($data) {
    if ($data && !empty($data['feature'])) {
      $features = $this->xmlParser->arrayNormalize($data['feature']);
      foreach ($features as $value) {
        $id = $value['Ид'];
        $type = $value['ТипЗначений'];
        $product = [];
        if (isset($value['ДляТоваров'])) {
          $product = $value['ДляТоваров'];
        }
        $feature = [
          'id' => $id,
          'name' => $value['Наименование'],
          'type' => $value['ТипЗначений'],
          'product' => $product,
          'props' => $this->featureProps($value),
        ];
        if ($type == 'Справочник') {
          $this->taxonomy[$id] = $feature;
        }
        else {
          $this->feature[$id] = $feature;
        }
      }
    }
  }

  /**
   * Feature Props.
   */
  private function featureProps($feature) {
    $props = [];
    if (!empty($feature['ВариантыЗначений']['Справочник'])) {
      $props = $this->xmlParser->xml2KeyVal($feature['ВариантыЗначений']['Справочник']);
    }
    return $props;
  }

}
